% function [U,S,V,relerrbnd,Jnum]=chebsvd(x)
%
% Computes the SVD of a Chebyshev-Vandermonde matrix
% to high relative accuracy.
% x=row vector of the nodes
%
% Reference: 
% James Demmel and Plamen Koev 
% "Accurate SVDs of polynomial Vandermonde matrices 
% involving orthonormal polynomials"
% Linear Algebra Appl., 417 (2006) 382-396.
% 
% Plamen Koev, 2001

function [U,S,V,relerrbnd,Jnum]=chebsvd(x)

n=size(x,2);
y=cos((2*(1:n)-1)*pi/2/n);

d1=ones(1,n); d2=d1;
for i=1:n
   d1(i)=prod(x(i)-y);
   for j=1:n
      if j~=i
         d2(i)=d2(i)/(y(i)-y(j));
      end
   end
end
[U,S,V,relerrbnd,Jnum] = cauchysvd1(x,y,d1,d2);
V=chebv(y)'*V;


