% function y=chebv(x)
% Given a row vector of nodes x returns the 
% Chebyshev-Vandermonde matrix evaluated at those nodes.
% Uses the three-term recurrence relationship.
%
% Reference: 
% James Demmel and Plamen Koev 
% "Accurate SVDs of polynomial Vandermonde matrices 
% involving orthonormal polynomials"
% Linear Algebra Appl., 417 (2006) 382-396.
% 
% Plamen Koev, 2001

function y=chebv(x)
n=size(x,2);
y=zeros(n);
y(:,1)=ones(n,1);
if n>0
   y(:,2)=x';
   for i=3:n 
       y(:,i)=2*x'.*y(:,i-1)-y(:,i-2);
   end
end
y=y/sqrt(n/2);
y(:,1)=y(:,1)/sqrt(2);
