% DGESV: computes singular value decomposition of the full column
%         rank matrix G, 
%
%                  G = U * S * V' 
%         
%         S is diagonal positive definite matrix whose diagonal
%         entries are the computed singular values.
%         U is orthogonal matrix (left singular vectors), and
%         V is orthogonal matrix (right singular vectors).
%
%         Jnum is number of Jacobi sweeps
%
%         The method uses one sided Jacobi type method. (from the right)
%         Only first rank(G) columns of U are computed.      
%         Standard precision (double). 
%
%  Written by I. Slapnicar, modified by J. Demmel
%    to avoid underflows
%
% function [U,S,V,Jnum] = dgesv2(G)
function [retval1, retval2, retval3, Jnum]=dgesv2(G)
[m,n]=size(G);
cycle=0;
convergence=0;
V=eye(n);
tol=m*eps;
db = 0;

% main loop

% figure(1), hold off, clf
%          figure(2)
%          GG = G'*G; dGG = diag(1./sqrt(diag(GG)));sGG = dGG*GG*dGG;
%          spy_mat1(max(sGG,1e-16),0)
% pause
while convergence<1,
Gold = G;
contest = 0;
count = 0;
saverots=[];
saverots1=zeros(n,n);
cycle=cycle+1;
convergence=2;

% Refreshing the diagonal of G'*G

for i=1:n; diagH(i)=norm(G(:,i))^2; end;

% row cyclic strategy

   for i=1:n-1
      for j=i+1:n
%           G'*G,           pause
         a=diagH(i);
         b=diagH(j);
         c=G(:,i)'*G(:,j);
    if (a==0 | b==0),
        i,j,a,b,c, 
        ngi=max(abs(G(:,i))), ngi*norm(G(:,i)/ngi)
        ngj=max(abs(G(:,j))), ngj*norm(G(:,j)/ngj)
        pause
    end
% Print intermediate results
     if (db==1)
         disp('cycle,i,j=')
         [cycle,i,j]
         disp('a,b,c=')
         [a,b,c]
         pause
     end

% Test wheteher to rotate. The program stops after an empty cycle.

         ttt = abs(c)/sqrt(abs(a))/sqrt(abs(b));
         contest = max( contest, ttt );
         saverots = [saverots;ttt];
         if abs(c) > tol * sqrt(abs(a)) * sqrt(abs(b));             
            saverots1(i,j)=ttt;
            convergence=0;
            count = count + 1;

% Computing the rotation matrix
%     [ cs sn1 ] [ a c ] [ cs  sn ] = [ a' 0 ]
%     [ sn  cs ] [ c b ] [ sn1 cs ]   [ 0  b']

            zeta=(b-a)/(2*abs(c));
            if (zeta==0),
               sig=1;
            else
               sig=sign(zeta);
            end;
%           t=sig/(abs(zeta)+sqrt(zeta^2+1));
            mxz = max(abs(zeta),1);
            mnz = min(abs(zeta),1);
            t=sig/(abs(zeta)+mxz*sqrt((mnz/mxz)^2+1));
            h=sqrt(1+t^2);
            cs=1/h;
            sn=t/h;
            qq=sign(c');

%%%% Updating the diagonal elements of G'*G
%%%         diagH(i)=a-c*t;
%%%         diagH(j)=b+c*t;

% Updating the i-th and j-th colum of G
 
            oldG = G;
            tmp=G(:,i);
            G(:,i)=cs*tmp-sn*qq*G(:,j);
            G(:,j)=sn*tmp+cs*qq*G(:,j);

%             Q=[cs -sn*qq; sn cs*qq]; norm(Q'*Q-eye(2))
            
%%%% Updating the diagonal elements of G'*G
             diagH(i)=norm(G(:,i))^2;
             diagH(j)=norm(G(:,j))^2;

% Updating the matrix V (right singular vectors)

            tmp=V(:,i);
            V(:,i)=cs*tmp-sn*qq*V(:,j);
            V(:,j)=sn*tmp+cs*qq*V(:,j);
            
%            G*V'
% Print intermediate results
        if (db==1 & cycle > 1)
            disp('cycle,count,i,j,a,b,c,zeta,t,h,sn,cs=')
            [cycle,count,i,j,a,b,c,zeta,t,h,sn,cs]',
        end

         end; % if
      end; % j
   end; % i
%        disp('cycle,contest, count='), [cycle,contest,count,count/(n*(n-1)/2)]
%        saverots = sort(saverots);
%        figure(1);
%        hold off, clf, 
%        semilogy(saverots); title(['cycle = ',int2str(cycle)])
%        axis([0,length(saverots),1e-16,1]) 
%        hold on, semilogy([1,length(saverots)],[tol,tol],'r'),
%        figure(2)
%        GG = G'*G; dGG = diag(1./sqrt(diag(GG)));sGG = dGG*GG*dGG;
%        spy_mat1(max(sGG,1e-16),0)
%        figure(3)
%        spy_mat1(max(sGG,1e-16),1)
%        figure(5), spy_mat1(saverots1,0)
%        pause
       if (db==1),
         disp('cycle,contest=')
         [cycle,contest]
            XX = Gold'*Gold;
            dXX = diag(1./sqrt(diag(XX)));
            sXX = abs(dXX*XX*dXX);
            figure(1),hold off, clg, spy_mat1(sXX,0);
            XXX = G'*G;
            dXXX = diag(1./sqrt(diag(XXX)));
            sXXX = abs(dXXX*XXX*dXXX);
            figure(2),hold off, clg, spy_mat1(sXXX,0);
            colors = ['m' 'b' 'c' 'g' 'y' 'r'];
            disp('pause'), pause
       end
end; % while
% Print intermediate results
GG = G'*G;
% DGG = inv(diag(sqrt(diag(GG))));
DGG = diag(1./sqrt(diag(GG)));
Conv = DGG*GG*DGG;
for i=1:n; 
   S(i)=norm(G(:,i));
   G(:,i)=G(:,i)/S(i); 
end;
cycle;
[sS,is] = sort(-S);
sS = -sS;
S=diag(sS);
V = V(:,is);
G = G(:,is);
retval1=G;           
retval2=S;
retval3=V;
Jnum = cycle;
